@@ -32,7 +32,7 @@ The standard way to use memcache with a database is like this::
     obj = mc.get(key)
     if not obj:
         obj = backend_api.get(...)
-        mc.set(obj)
+        mc.set(key, obj)
 
     # we now have obj, and future passes through this code
     # will use the object from the cache.
@@ -69,7 +69,7 @@ from binascii import crc32   # zlib version is not cross-platform
 serverHashFunction = crc32
 
 __author__    = "Evan Martin <martine@danga.com>"
-__version__ = "1.44"
+__version__ = "1.44.1"
 __copyright__ = "Copyright (C) 2003 Danga Interactive"
 __license__   = "Python"
 
@@ -367,7 +367,7 @@ class Client(local):
             cmd = "delete %s" % key
 
         try:
-            server.send_cmd(cmd)
+            server.send_cmd(cmd.encode('utf-8'))
             server.expect(b"DELETED")
         except socket.error as msg:
             if type(msg) is tuple: msg = msg[1]
@@ -377,12 +377,13 @@ class Client(local):
 
     def incr(self, key, delta=1):
         """
-        Sends a command to the server to atomically increment the value for C{key} by
-        C{delta}, or by 1 if C{delta} is unspecified.  Returns None if C{key} doesn't
-        exist on server, otherwise it returns the new value after incrementing.
+        Sends a command to the server to atomically increment the value
+        for C{key} by C{delta}, or by 1 if C{delta} is unspecified.
+        Returns None if C{key} doesn't exist on server, otherwise it
+        returns the new value after incrementing.
 
-        Note that the value for C{key} must already exist in the memcache, and it
-        must be the string representation of an integer.
+        Note that the value for C{key} must already exist in the memcache,
+        and it must be the string representation of an integer.
 
         >>> mc.set("counter", "20")  # returns 1, indicating success
         1
@@ -422,9 +423,10 @@ class Client(local):
         try:
             server.send_cmd(cmd)
             line = server.readline()
+            if line == None or line.strip() ==b'NOT_FOUND': return None
             return int(line)
         except socket.error as msg:
-            if type(msg) is tuple: msg = msg[1]
+            if isinstance(msg, tuple): msg = msg[1]
             server.mark_dead(msg)
             return None
 
@@ -589,13 +591,19 @@ class Client(local):
         dead_servers = []
 
         for server in server_keys.keys():
-            bigcmd = []
-            write = bigcmd.append
+            bigcmd = bytearray()
+            write = bigcmd.extend
             try:
+                newline = "\r\n".encode('utf-8')
                 for key in server_keys[server]: # These are mangled keys
                     store_info = self._val_to_store_info(mapping[prefixed_to_orig_key[key]], min_compress_len)
-                    write("set %s %d %d %d\r\n%s\r\n" % (key, store_info[0], time, store_info[1], store_info[2]))
-                server.send_cmds(''.join(bigcmd))
+                    cmd = ("set %s %d %d %d\r\n" % (key, store_info[0], time, store_info[1])).encode('utf-8')
+                    if not isinstance(store_info[2],bytes):
+                        cmd += store_info[2].encode('utf-8')
+                    else:
+                        cmd += store_info[2]
+                    write(cmd + newline)
+                server.send_cmds(bytes(bigcmd))
             except socket.error as msg:
                 if type(msg) is tuple: msg = msg[1]
                 server.mark_dead(msg)
@@ -613,7 +621,7 @@ class Client(local):
             try:
                 for key in keys:
                     line = server.readline()
-                    if line == 'STORED':
+                    if line == b'STORED':
                         continue
                     else:
                         notstored.append(prefixed_to_orig_key[key]) #un-mangle.
@@ -627,18 +635,17 @@ class Client(local):
            Transform val to a storable representation, returning a tuple of the flags, the length of the new value, and the new value itself.
         """
         flags = 0
-        if isinstance(val, str):
+
+        # check types exactly rather than using isinstance, or subclasses
+        # will be deserialized into instances of the parent class
+        # (most blatantly, bool --> int)
+        if type(val) == str:
             pass
-        elif isinstance(val, int):
+        elif type(val) == int:
             flags |= Client._FLAG_INTEGER
             val = "%d" % val
             # force no attempt to compress this silly string.
             min_compress_len = 0
-        elif isinstance(val, int):
-            flags |= Client._FLAG_LONG
-            val = "%d" % val
-            # force no attempt to compress this silly string.
-            min_compress_len = 0
         else:
             flags |= Client._FLAG_PICKLE
             file = BytesIO()
@@ -922,7 +929,7 @@ class _Host:
             self.socket.sendall((cmd + '\r\n').encode('ascii'))
         else:
             self.socket.sendall(cmd + '\r\n'.encode('ascii'))
-        
+
 
     def send_cmds(self, cmds):
         """ cmds already has trailing \r\n's applied """
@@ -1032,7 +1039,7 @@ if __name__ == "__main__":
             print("Testing set/get {'%s': %s} ..." % (to_s(key), to_s(val)), end=' ')
             mc.set(key, val)
             newval = mc.get(key)
-            if newval == val:
+            if newval == val and type(newval) == type(val):
                 print("OK")
                 return 1
             else:
@@ -1050,8 +1057,14 @@ if __name__ == "__main__":
                     return self.bar == other.bar
                 return 0
 
+        class StrSubclass(str):
+            pass
+
         test_setget("a_string", "some random string")
+        test_setget("a_string_subclass", StrSubclass("L337 57R"))
         test_setget("an_integer", 42)
+        test_setget("bool_True", True)
+        test_setget("bool_False", False)
         if test_setget("long", int(1<<30)):
             print("Testing delete ...", end=' ')
             if mc.delete("long"):
